@@ -27,10 +27,15 @@
 
 #include "config.h"
 #include <string.h>
+#include <math.h>
 #include <glib/gi18n-lib.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
+#ifdef HAVE_WEBKIT2
+#include <webkit2/webkit2.h>
+#else
 #include <webkit/webkit.h>
+#endif
 
 #ifdef GDK_WINDOWING_QUARTZ
 #include <gtkosxapplication.h>
@@ -92,23 +97,25 @@ static const
 struct
 {
         gchar *name;
-        int    level;
+        double level;
 }
 zoom_levels[] =
 {
-        { N_("50%"), 70 },
-        { N_("75%"), 84 },
-        { N_("100%"), 100 },
-        { N_("125%"), 119 },
-        { N_("150%"), 141 },
-        { N_("175%"), 168 },
-        { N_("200%"), 200 },
-        { N_("300%"), 283 },
-        { N_("400%"), 400 }
+        { N_("50%"), 0.5 },
+        { N_("75%"), 0.8408964152 },
+        { N_("100%"), 1.0 },
+        { N_("125%"), 1.1892071149 },
+        { N_("150%"), 1.4142135623 },
+        { N_("175%"), 1.6817928304 },
+        { N_("200%"), 2.0 },
+        { N_("300%"), 2.8284271247 },
+        { N_("400%"), 4.0 }
 };
 
+static const guint n_zoom_levels = G_N_ELEMENTS (zoom_levels);
+
 #define ZOOM_MINIMAL    (zoom_levels[0].level)
-#define ZOOM_MAXIMAL    (zoom_levels[8].level)
+#define ZOOM_MAXIMAL    (zoom_levels[n_zoom_levels - 1].level)
 #define ZOOM_DEFAULT    (zoom_levels[2].level)
 
 static void           dh_window_class_init           (DhWindowClass   *klass);
@@ -155,6 +162,7 @@ static void           window_tab_set_title           (DhWindow        *window,
                                                       const gchar     *title);
 static void           window_close_tab               (DhWindow *window,
                                                       gint      page_num);
+static gboolean       do_search                      (DhWindow *window);
 
 G_DEFINE_TYPE (DhWindow, dh_window, GTK_TYPE_WINDOW);
 
@@ -189,10 +197,16 @@ static void
 window_activate_print (GtkAction *action,
                        DhWindow  *window)
 {
-    WebKitWebView *web_view;
+        WebKitWebView *web_view = window_get_active_web_view (window);
+#ifdef HAVE_WEBKIT2
+        WebKitPrintOperation *print_operation;
 
-    web_view = window_get_active_web_view (window);
-    webkit_web_view_execute_script (web_view, "print();");
+        print_operation = webkit_print_operation_new (web_view);
+        webkit_print_operation_run_dialog (print_operation, GTK_WIDGET (window));
+        g_object_unref (print_operation);
+#else
+        webkit_web_view_execute_script (web_view, "print();");
+#endif
 }
 
 static void
@@ -257,7 +271,11 @@ window_activate_copy (GtkAction *action,
                 WebKitWebView *web_view;
 
                 web_view = window_get_active_web_view (window);
+#ifdef HAVE_WEBKIT2
+                webkit_web_view_execute_editing_command (web_view, WEBKIT_EDITING_COMMAND_COPY);
+#else
                 webkit_web_view_copy_clipboard (web_view);
+#endif
         }
 }
 
@@ -266,36 +284,49 @@ window_activate_find (GtkAction *action,
                       DhWindow  *window)
 {
         DhWindowPriv  *priv;
+#ifndef HAVE_WEBKIT2
         WebKitWebView *web_view;
-
+#endif
         priv = window->priv;
-        web_view = window_get_active_web_view (window);
 
         gtk_widget_show (priv->findbar);
         gtk_widget_grab_focus (priv->findbar);
 
+#ifdef HAVE_WEBKIT2
+        /* The behaviour for WebKit1 is to re-enable highlighting without
+           starting a new search. WebKit2 API does not allow that
+           without invoking a new search. */
+        do_search (window);
+#else
+        web_view = window_get_active_web_view (window);
         webkit_web_view_set_highlight_text_matches (web_view, TRUE);
+#endif /* HAVE_WEBKIT2 */
 }
 
 static int
 window_get_current_zoom_level_index (DhWindow *window)
 {
         WebKitWebView *web_view;
-        float zoom_level;
-        int zoom_level_as_int = ZOOM_DEFAULT;
+        double previous, current, mean;
+        double zoom_level = ZOOM_DEFAULT;
         int i;
 
         web_view = window_get_active_web_view (window);
-        if (web_view) {
-                g_object_get (web_view, "zoom-level", &zoom_level, NULL);
-                zoom_level_as_int = (int)(zoom_level*100);
+        if (web_view)
+                zoom_level = webkit_web_view_get_zoom_level (web_view);
+
+        previous = zoom_levels[0].level;
+        for (i = 1; i < n_zoom_levels; i++) {
+                current = zoom_levels[i].level;
+                mean = sqrt (previous * current);
+
+                if (zoom_level <= mean)
+                        return i - 1;
+
+                previous = current;
         }
 
-        for (i=0; zoom_levels[i].level != ZOOM_MAXIMAL; i++) {
-                if (zoom_levels[i].level == zoom_level_as_int)
-                        return i;
-        }
-        return i;
+        return n_zoom_levels - 1;
 }
 
 static void
@@ -334,9 +365,7 @@ window_activate_zoom_in (GtkAction *action,
                 WebKitWebView *web_view;
 
                 web_view = window_get_active_web_view (window);
-                g_object_set (web_view,
-                              "zoom-level", (float)(zoom_levels[zoom_level_idx+1].level)/100,
-                              NULL);
+                webkit_web_view_set_zoom_level (web_view, zoom_levels[zoom_level_idx + 1].level);
                 window_update_zoom_actions_sensitiveness (window);
         }
 
@@ -356,9 +385,7 @@ window_activate_zoom_out (GtkAction *action,
                 WebKitWebView *web_view;
 
                 web_view = window_get_active_web_view (window);
-                g_object_set (web_view,
-                              "zoom-level", (float)(zoom_levels[zoom_level_idx-1].level)/100,
-                              NULL);
+                webkit_web_view_set_zoom_level (web_view, zoom_levels[zoom_level_idx - 1].level);
                 window_update_zoom_actions_sensitiveness (window);
         }
 }
@@ -373,7 +400,7 @@ window_activate_zoom_default (GtkAction *action,
         priv = window->priv;
 
         web_view = window_get_active_web_view (window);
-        g_object_set (web_view, "zoom-level", (float)(ZOOM_DEFAULT)/100, NULL);
+        webkit_web_view_set_zoom_level (web_view, ZOOM_DEFAULT);
         window_update_zoom_actions_sensitiveness (window);
 }
 
@@ -1017,14 +1044,12 @@ window_web_view_switch_page_cb (GtkNotebook     *notebook,
         new_page = gtk_notebook_get_nth_page (notebook, new_page_num);
         if (new_page) {
                 WebKitWebView  *new_web_view;
-                WebKitWebFrame *web_frame;
                 const gchar    *location;
 
                 new_web_view = g_object_get_data (G_OBJECT (new_page), "web_view");
 
                 /* Sync the book tree. */
-                web_frame = webkit_web_view_get_main_frame (new_web_view);
-                location = webkit_web_frame_get_uri (web_frame);
+                location = webkit_web_view_get_uri (new_web_view);
 
                 if (location) {
                         dh_book_tree_select_uri (DH_BOOK_TREE (priv->book_tree),
@@ -1272,7 +1297,13 @@ find_library_equivalent (DhWindow    *window,
         return local_uri;
 }
 
-
+#ifdef HAVE_WEBKIT2
+static gboolean
+window_web_view_decide_policy_cb (WebKitWebView           *web_view,
+                                  WebKitPolicyDecision    *policy_decision,
+                                  WebKitPolicyDecisionType type,
+                                  DhWindow                *window)
+#else
 static gboolean
 window_web_view_navigation_policy_decision_requested (WebKitWebView             *web_view,
                                                       WebKitWebFrame            *frame,
@@ -1280,19 +1311,36 @@ window_web_view_navigation_policy_decision_requested (WebKitWebView
                                                       WebKitWebNavigationAction *navigation_action,
                                                       WebKitWebPolicyDecision   *policy_decision,
                                                       DhWindow                  *window)
+#endif
 {
         DhWindowPriv *priv;
         const char   *uri;
+#ifdef HAVE_WEBKIT2
+        WebKitNavigationPolicyDecision *navigation_decision;
+#endif
 
         priv = window->priv;
 
+#ifdef HAVE_WEBKIT2
+        if (type != WEBKIT_POLICY_DECISION_TYPE_NAVIGATION_ACTION)
+                return FALSE;
+
+        navigation_decision = WEBKIT_NAVIGATION_POLICY_DECISION (policy_decision);
+        uri = webkit_uri_request_get_uri (webkit_navigation_policy_decision_get_request (navigation_decision));
+#else
         uri = webkit_network_request_get_uri (request);
+#endif
 
         /* make sure to hide the info bar on page change */
         gtk_widget_hide (window_get_active_info_bar (window));
 
+#ifdef HAVE_WEBKIT2
+        if (webkit_navigation_policy_decision_get_mouse_button (navigation_decision) == 2) { /* middle click */
+                webkit_policy_decision_ignore (policy_decision);
+#else
         if (webkit_web_navigation_action_get_button (navigation_action) == 2) { /* middle click */
                 webkit_web_policy_decision_ignore (policy_decision);
+#endif
                 g_signal_emit (window, signals[OPEN_LINK], 0, uri, DH_OPEN_LINK_NEW_TAB);
                 return TRUE;
         }
@@ -1304,7 +1352,11 @@ window_web_view_navigation_policy_decision_requested (WebKitWebView
         if (strncmp (uri, "http://library.gnome.org/devel/", 31) == 0) {
                 gchar *local_uri = find_library_equivalent (window, uri);
                 if (local_uri) {
+#ifdef HAVE_WEBKIT2
+                        webkit_policy_decision_ignore (policy_decision);
+#else
                         webkit_web_policy_decision_ignore (policy_decision);
+#endif
                         _dh_window_display_uri (window, local_uri);
                         g_free (local_uri);
                         return TRUE;
@@ -1312,7 +1364,11 @@ window_web_view_navigation_policy_decision_requested (WebKitWebView
         }
 
         if (strncmp (uri, "file://", 7) != 0) {
+#ifdef HAVE_WEBKIT2
+                webkit_policy_decision_ignore (policy_decision);
+#else
                 webkit_web_policy_decision_ignore (policy_decision);
+#endif
                 gtk_show_uri (NULL, uri, GDK_CURRENT_TIME, NULL);
                 return TRUE;
         }
@@ -1325,13 +1381,21 @@ window_web_view_navigation_policy_decision_requested (WebKitWebView
         return FALSE;
 }
 
-
+#ifdef HAVE_WEBKIT2
+static gboolean
+window_web_view_load_failed_cb (WebKitWebView   *web_view,
+                                WebKitLoadEvent  load_event,
+                                const gchar     *uri,
+                                GError          *web_error,
+                                DhWindow        *window)
+#else
 static gboolean
 window_web_view_load_error_cb (WebKitWebView  *web_view,
                                WebKitWebFrame *frame,
                                gchar          *uri,
                                gpointer       *web_error,
                                DhWindow       *window)
+#endif
 {
         GtkWidget *info_bar;
         GtkWidget *content_area;
@@ -1423,11 +1487,12 @@ window_check_history (DhWindow      *window,
 }
 
 static void
-window_web_view_title_changed_cb (WebKitWebView  *web_view,
-                                  WebKitWebFrame *web_frame,
-                                  const gchar    *title,
-                                  DhWindow       *window)
+window_web_view_title_changed_cb (WebKitWebView *web_view,
+                                  GParamSpec    *param_spec,
+                                  DhWindow      *window)
 {
+        const gchar *title = webkit_web_view_get_title (web_view);
+
         if (web_view == window_get_active_web_view (window)) {
                 window_update_title (window, web_view, title);
         }
@@ -1450,10 +1515,20 @@ window_web_view_button_press_event_cb (WebKitWebView  *web_view,
 static gboolean
 do_search (DhWindow *window)
 {
-        DhWindowPriv  *priv = window->priv;
-        WebKitWebView *web_view;
+        DhWindowPriv         *priv = window->priv;
+#ifdef HAVE_WEBKIT2
+        WebKitFindController *find_controller;
+        guint                 find_options = WEBKIT_FIND_OPTIONS_WRAP_AROUND;
+        const gchar          *search_text;
 
-        priv->find_source_id = 0;
+        find_controller = webkit_web_view_get_find_controller (window_get_active_web_view (window));
+        if (!egg_find_bar_get_case_sensitive (EGG_FIND_BAR (priv->findbar)))
+                find_options |= WEBKIT_FIND_OPTIONS_CASE_INSENSITIVE;
+
+        search_text = egg_find_bar_get_search_string (EGG_FIND_BAR (priv->findbar));
+        webkit_find_controller_search (find_controller, search_text, find_options, G_MAXUINT);
+#else
+        WebKitWebView *web_view;
 
         web_view = window_get_active_web_view (window);
 
@@ -1468,6 +1543,9 @@ do_search (DhWindow *window)
                 web_view, egg_find_bar_get_search_string (EGG_FIND_BAR (priv->findbar)),
                 egg_find_bar_get_case_sensitive (EGG_FIND_BAR (priv->findbar)),
                 TRUE, TRUE);
+#endif /* HAVE_WEBKIT2 */
+
+        priv->find_source_id = 0;
 
 	return FALSE;
 }
@@ -1492,6 +1570,9 @@ window_find_case_changed_cb (GObject    *object,
                              GParamSpec *pspec,
                              DhWindow   *window)
 {
+#ifdef HAVE_WEBKIT2
+        do_search (window);
+#else
         DhWindowPriv  *priv = window->priv;;
         WebKitWebView *view;
         const gchar   *string;
@@ -1505,58 +1586,78 @@ window_find_case_changed_cb (GObject    *object,
         webkit_web_view_unmark_text_matches (view);
         webkit_web_view_mark_text_matches (view, string, case_sensitive, 0);
         webkit_web_view_set_highlight_text_matches (view, TRUE);
+#endif
 }
 
 static void
 window_find_next_cb (GtkEntry *entry,
                      DhWindow *window)
 {
-        DhWindowPriv  *priv = window->priv;
-        WebKitWebView *view;
-        const gchar   *string;
-        gboolean       case_sensitive;
-
+        DhWindowPriv         *priv = window->priv;
+        WebKitWebView        *view;
+#ifdef HAVE_WEBKIT2
+        WebKitFindController *find_controller;
+#else
+        const gchar          *string;
+        gboolean              case_sensitive;
+#endif
         view = window_get_active_web_view (window);
 
         gtk_widget_show (priv->findbar);
-
+#ifdef HAVE_WEBKIT2
+        find_controller = webkit_web_view_get_find_controller (view);
+        webkit_find_controller_search_next(find_controller);
+#else
         string = egg_find_bar_get_search_string (EGG_FIND_BAR (priv->findbar));
         case_sensitive = egg_find_bar_get_case_sensitive (EGG_FIND_BAR (priv->findbar));
-
         webkit_web_view_search_text (view, string, case_sensitive, TRUE, TRUE);
+#endif
 }
 
 static void
 window_find_previous_cb (GtkEntry *entry,
                          DhWindow *window)
 {
-        DhWindowPriv  *priv = window->priv;
-        WebKitWebView *view;
-        const gchar   *string;
-        gboolean       case_sensitive;
-
+        DhWindowPriv         *priv = window->priv;
+        WebKitWebView        *view;
+#ifdef HAVE_WEBKIT2
+        WebKitFindController *find_controller;
+#else
+        const gchar          *string;
+        gboolean             case_sensitive;
+#endif
         view = window_get_active_web_view (window);
 
         gtk_widget_show (priv->findbar);
 
+#ifdef HAVE_WEBKIT2
+        find_controller = webkit_web_view_get_find_controller (view);
+        webkit_find_controller_search_previous(find_controller);
+#else
         string = egg_find_bar_get_search_string (EGG_FIND_BAR (priv->findbar));
         case_sensitive = egg_find_bar_get_case_sensitive (EGG_FIND_BAR (priv->findbar));
-
         webkit_web_view_search_text (view, string, case_sensitive, FALSE, TRUE);
+#endif
 }
 
 static void
 window_findbar_close_cb (GtkWidget *widget,
                          DhWindow  *window)
 {
-        DhWindowPriv  *priv = window->priv;
-        WebKitWebView *view;
-
+        DhWindowPriv         *priv = window->priv;
+        WebKitWebView        *view;
+#ifdef HAVE_WEBKIT2
+        WebKitFindController *find_controller;
+#endif
         view = window_get_active_web_view (window);
 
         gtk_widget_hide (priv->findbar);
-
+#ifdef HAVE_WEBKIT2
+        find_controller = webkit_web_view_get_find_controller (view);
+        webkit_find_controller_search_finish (find_controller);
+#else
         webkit_web_view_set_highlight_text_matches (view, FALSE);
+#endif
 }
 
 #if 0
@@ -1603,10 +1704,12 @@ window_open_new_tab (DhWindow    *window,
         DhWindowPriv *priv;
         GtkWidget    *view;
         GtkWidget    *vbox;
-        GtkWidget    *scrolled_window;
         GtkWidget    *label;
         gint          num;
         GtkWidget    *info_bar;
+#ifndef HAVE_WEBKIT2
+        GtkWidget    *scrolled_window;
+#endif
 
         priv = window->priv;
 
@@ -1646,6 +1749,9 @@ window_open_new_tab (DhWindow    *window,
 
         gtk_box_pack_start (GTK_BOX(vbox), info_bar, FALSE, TRUE, 0);
 
+#ifdef HAVE_WEBKIT2
+        gtk_box_pack_start (GTK_BOX(vbox), view, TRUE, TRUE, 0);
+#else
         scrolled_window = gtk_scrolled_window_new (NULL, NULL);
         gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                         GTK_POLICY_AUTOMATIC,
@@ -1653,22 +1759,35 @@ window_open_new_tab (DhWindow    *window,
         gtk_container_add (GTK_CONTAINER (scrolled_window), view);
         gtk_widget_show (scrolled_window);
         gtk_box_pack_start (GTK_BOX(vbox), scrolled_window, TRUE, TRUE, 0);
+#endif
 
         label = window_new_tab_label (window, _("Empty Page"), vbox);
         gtk_widget_show_all (label);
 
-        g_signal_connect (view, "title-changed",
+        g_signal_connect (view, "notify::title",
                           G_CALLBACK (window_web_view_title_changed_cb),
                           window);
         g_signal_connect (view, "button-press-event",
                           G_CALLBACK (window_web_view_button_press_event_cb),
                           window);
+#ifdef HAVE_WEBKIT2
+        g_signal_connect (view, "decide-policy",
+                          G_CALLBACK (window_web_view_decide_policy_cb),
+                          window);
+#else
         g_signal_connect (view, "navigation-policy-decision-requested",
                           G_CALLBACK (window_web_view_navigation_policy_decision_requested),
                           window);
+#endif
+#ifdef HAVE_WEBKIT2
+        g_signal_connect (view, "load-failed",
+                          G_CALLBACK (window_web_view_load_failed_cb),
+                          window);
+#else
         g_signal_connect (view, "load-error",
                           G_CALLBACK (window_web_view_load_error_cb),
                           window);
+#endif
 
         num = gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook),
                                         vbox, NULL);
@@ -1844,12 +1963,8 @@ window_update_title (DhWindow      *window,
 
         priv = window->priv;
 
-        if (!web_view_title) {
-                WebKitWebFrame *web_frame;
-
-                web_frame = webkit_web_view_get_main_frame (web_view);
-                web_view_title = webkit_web_frame_get_title (web_frame);
-        }
+        if (!web_view_title)
+                web_view_title = webkit_web_view_get_title (web_view);
 
         if (web_view_title && *web_view_title == '\0') {
                 web_view_title = NULL;
